@@ -17,6 +17,7 @@
  */
 
 #include "gtd-provider-eds.h"
+#include "gtd-task-eds.h"
 #include "gtd-task-list-eds.h"
 
 #include <glib/gi18n.h>
@@ -73,6 +74,21 @@ task_data_new (GtdProviderEds *provider,
   return tdata;
 }
 
+static void
+ensure_offline_sync (GtdProviderEds *self,
+                     ESource        *source)
+{
+  GtdProviderEdsPrivate *priv;
+  ESourceOffline *extension;
+
+  priv = gtd_provider_eds_get_instance_private (self);
+
+  extension = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
+  e_source_offline_set_stay_synchronized (extension, TRUE);
+
+  e_source_registry_commit_source (priv->source_registry, source, NULL, NULL, NULL);
+}
+
 static void
 gtd_provider_eds_set_default (GtdProviderEds *self,
                               GtdTaskList    *list)
@@ -121,7 +137,7 @@ gtd_provider_eds_fill_task_list (GObject      *client,
         {
           GtdTask *task;
 
-          task = gtd_task_new (l->data);
+          task = gtd_task_eds_new (l->data);
           gtd_task_set_list (task, list);
 
           gtd_task_list_save_task (list, task);
@@ -191,6 +207,9 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
   /* creates a new task list */
   list = gtd_task_list_eds_new (GTD_PROVIDER (self), source);
 
+  /* ensure tasklist is cached offline */
+  ensure_offline_sync (self, source);
+
   /* it's not ready until we fetch the list of tasks from client */
   gtd_object_set_ready (GTD_OBJECT (list), FALSE);
 
@@ -879,7 +898,7 @@ gtd_provider_eds_create_task (GtdProviderEds *provider,
   tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
   source = gtd_task_list_eds_get_source (tasklist);
   client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_get_component (task);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
 
   /* Temporary data for async operation */
   data = task_data_new (provider, (gpointer) task);
@@ -912,7 +931,9 @@ gtd_provider_eds_update_task (GtdProviderEds *provider,
   tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
   source = gtd_task_list_eds_get_source (tasklist);
   client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_get_component (task);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
+
+  e_cal_component_commit_sequence (component);
 
   /* Temporary data for async operation */
   data = task_data_new (provider, (gpointer) task);
@@ -948,7 +969,7 @@ gtd_provider_eds_remove_task (GtdProviderEds *provider,
   tasklist = GTD_TASK_LIST_EDS (gtd_task_get_list (task));
   source = gtd_task_list_eds_get_source (tasklist);
   client = g_hash_table_lookup (priv->clients, source);
-  component = gtd_task_get_component (task);
+  component = gtd_task_eds_get_component (GTD_TASK_EDS (task));
   id = e_cal_component_get_id (component);
 
   /* Temporary data for async operation */
@@ -1099,3 +1120,19 @@ gtd_provider_eds_set_default_task_list (GtdProviderEds *provider,
 
   g_object_notify (G_OBJECT (provider), "default-task-list");
 }
+
+GtdTask*
+gtd_provider_eds_generate_task (GtdProviderEds *self)
+{
+  ECalComponent *component;
+  GtdTask *task;
+
+  component = e_cal_component_new ();
+  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
+
+  task = gtd_task_eds_new (component);
+
+  g_object_unref (component);
+
+  return task;
+}
