@@ -84,12 +84,6 @@ gtd_provider_local_get_icon (GtdProvider *provider)
   return self->icon;
 }
 
-static const GtkWidget*
-gtd_provider_local_get_edit_panel (GtdProvider *provider)
-{
-  return NULL;
-}
-
 static void
 gtd_provider_local_create_task (GtdProvider *provider,
                                 GtdTask     *task)
@@ -196,6 +190,12 @@ gtd_provider_local_set_default_task_list (GtdProvider *provider,
   gtd_provider_eds_set_default_task_list (GTD_PROVIDER_EDS (provider), list);
 }
 
+static GtdTask*
+gtd_provider_local_generate_task (GtdProvider *provider)
+{
+  return gtd_provider_eds_generate_task (GTD_PROVIDER_EDS (provider));
+}
+
 static void
 gtd_provider_iface_init (GtdProviderInterface *iface)
 {
@@ -204,7 +204,6 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
   iface->get_description = gtd_provider_local_get_description;
   iface->get_enabled = gtd_provider_local_get_enabled;
   iface->get_icon = gtd_provider_local_get_icon;
-  iface->get_edit_panel = gtd_provider_local_get_edit_panel;
   iface->create_task = gtd_provider_local_create_task;
   iface->update_task = gtd_provider_local_update_task;
   iface->remove_task = gtd_provider_local_remove_task;
@@ -214,6 +213,7 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
   iface->get_task_lists = gtd_provider_local_get_task_lists;
   iface->get_default_task_list = gtd_provider_local_get_default_task_list;
   iface->set_default_task_list = gtd_provider_local_set_default_task_list;
+  iface->generate_task = gtd_provider_local_generate_task;
 }
 
 GtdProviderLocal*
