@@ -19,9 +19,9 @@
 #include "gtd-provider-todo-txt.h"
 #include "gtd-plugin-todo-txt.h"
 #include "gtd-todo-txt-parser.h"
+
 #include <string.h>
 #include <stdlib.h>
-
 #include <glib/gi18n.h>
 
 
@@ -98,12 +98,6 @@ gtd_provider_todo_txt_get_icon (GtdProvider *provider)
   return self->icon;
 }
 
-static const GtkWidget*
-gtd_provider_todo_txt_get_edit_panel (GtdProvider *provider)
-{
-  return NULL;
-}
-
 static void
 emit_generic_error (GError *error)
 {
@@ -205,22 +199,6 @@ create_list (GtdProviderTodoTxt *self,
   return task_list;
 }
 
-GtdTask*
-create_task (void)
-{
-  ECalComponent *component;
-  GtdTask *task;
-
-  component = e_cal_component_new ();
-
-  e_cal_component_set_new_vtype (component, E_CAL_COMPONENT_TODO);
-  e_cal_component_set_uid (component, e_cal_component_gen_uid ());
-
-  task = gtd_task_new (component);
-
-  return task;
-}
-
 static void
 gtd_provider_todo_txt_load_tasks (GtdProviderTodoTxt *self)
 {
@@ -290,8 +268,11 @@ gtd_provider_todo_txt_load_tasks (GtdProviderTodoTxt *self)
               continue;
             }
 
-          task = gtd_todo_txt_parser_parse_tokens (tokens);
+          task = gtd_provider_generate_task (GTD_PROVIDER (self));
+          gtd_todo_txt_parser_parse_tokens (task, tokens);
+
           g_hash_table_insert (self->tasks, g_strdup (gtd_task_get_title (task)), task);
+
           list = create_list (self, g_object_get_data (G_OBJECT (task), "list_name"));
           gtd_task_set_list (task, list);
 
@@ -305,7 +286,7 @@ gtd_provider_todo_txt_load_tasks (GtdProviderTodoTxt *self)
                 }
               else
                 {
-                  parent_task = create_task ();
+                  parent_task = gtd_provider_generate_task (GTD_PROVIDER (self));
                   gtd_task_set_list (parent_task, list);
                   gtd_task_set_title (parent_task, g_object_get_data (G_OBJECT (task), "root_task_name"));
 
@@ -521,7 +502,6 @@ gtd_provider_iface_init (GtdProviderInterface *iface)
   iface->get_description = gtd_provider_todo_txt_get_description;
   iface->get_enabled = gtd_provider_todo_txt_get_enabled;
   iface->get_icon = gtd_provider_todo_txt_get_icon;
-  iface->get_edit_panel = gtd_provider_todo_txt_get_edit_panel;
   iface->create_task = gtd_provider_todo_txt_create_task;
   iface->update_task = gtd_provider_todo_txt_update_task;
   iface->remove_task = gtd_provider_todo_txt_remove_task;
