@@ -25,6 +25,7 @@
 #include "gtd-manager.h"
 #include "gtd-manager-protected.h"
 #include "gtd-plugin-dialog.h"
+#include "gtd-vcs-identifier.h"
 #include "gtd-window.h"
 
 #include <glib.h>
@@ -33,21 +34,14 @@
 #include <girepository.h>
 #include <glib/gi18n.h>
 
-typedef struct
-{
-  GtdManager     *manager;
-
-  GtkWidget      *window;
-  GtkWidget      *plugin_dialog;
-  GtkWidget      *initial_setup;
-} GtdApplicationPrivate;
 
 struct _GtdApplication
 {
   GtkApplication         application;
 
-  /*< private >*/
-  GtdApplicationPrivate *priv;
+  GtkWidget             *window;
+  GtkWidget             *plugin_dialog;
+  GtkWidget             *initial_setup;
 };
 
 static void           gtd_application_activate_action             (GSimpleAction        *simple,
@@ -70,7 +64,11 @@ static void           gtd_application_quit                        (GSimpleAction
                                                                    GVariant             *parameter,
                                                                    gpointer              user_data);
 
-G_DEFINE_TYPE_WITH_PRIVATE (GtdApplication, gtd_application, GTK_TYPE_APPLICATION)
+G_DEFINE_TYPE (GtdApplication, gtd_application, GTK_TYPE_APPLICATION)
+
+static GOptionEntry cmd_options[] = {
+  { "quit", 'q', 0, G_OPTION_ARG_NONE, NULL, N_("Quit GNOME To Do"), NULL }
+};
 
 static const GActionEntry gtd_application_entries[] = {
   { "activate", gtd_application_activate_action },
@@ -85,10 +83,10 @@ gtd_application_activate_action (GSimpleAction *simple,
                                  GVariant      *parameter,
                                  gpointer       user_data)
 {
-  GtdApplicationPrivate *priv = GTD_APPLICATION (user_data)->priv;
+  GtdApplication *self = GTD_APPLICATION (user_data);
 
-  gtk_widget_show (priv->window);
-  gtk_window_present (GTK_WINDOW (priv->window));
+  gtk_widget_show (self->window);
+  gtk_window_present (GTK_WINDOW (self->window));
 }
 
 static void
@@ -105,9 +103,9 @@ gtd_application_show_extensions (GSimpleAction *simple,
                                  GVariant      *parameter,
                                  gpointer       user_data)
 {
-  GtdApplicationPrivate *priv = GTD_APPLICATION (user_data)->priv;
+  GtdApplication *self = GTD_APPLICATION (user_data);
 
-  gtk_widget_show (priv->plugin_dialog);
+  gtk_widget_show (self->plugin_dialog);
 }
 
 static void
@@ -115,10 +113,10 @@ gtd_application_show_about (GSimpleAction *simple,
                             GVariant      *parameter,
                             gpointer       user_data)
 {
-  GtdApplicationPrivate *priv = GTD_APPLICATION (user_data)->priv;
-  char *copyright;
-  GDateTime *date;
-  int created_year = 2015;
+  g_autofree gchar *copyright = NULL;
+  g_autoptr (GDateTime) date = NULL;
+  GtdApplication *self;
+  gint created_year = 2015;
 
   static const gchar *authors[] = {
     "Emmanuele Bassi <ebassi@gnome.org>",
@@ -132,25 +130,27 @@ gtd_application_show_about (GSimpleAction *simple,
   static const gchar *artists[] = {
     "Allan Day <allanpday@gmail.com>",
     "Jakub Steiner <jimmac@gmail.com>",
+    "Tobias Bernard <tbernard@gnome.org>",
     NULL
   };
 
+  self = GTD_APPLICATION (user_data);
   date = g_date_time_new_now_local ();
 
   if (g_date_time_get_year (date) <= created_year)
     {
-      copyright = g_strdup_printf (_("Copyright \xC2\xA9 %d "
+      copyright = g_strdup_printf (_("Copyright \xC2\xA9 %1$d "
                                      "The To Do authors"), created_year);
     }
   else
     {
-      copyright = g_strdup_printf (_("Copyright \xC2\xA9 %d\xE2\x80\x93%d "
+      copyright = g_strdup_printf (_("Copyright \xC2\xA9 %1$d\xE2\x80\x93%2$d "
                                      "The To Do authors"), created_year, g_date_time_get_year (date));
     }
 
-  gtk_show_about_dialog (GTK_WINDOW (priv->window),
+  gtk_show_about_dialog (GTK_WINDOW (self->window),
                          "program-name", _("To Do"),
-                         "version", VERSION,
+                         "version", VERSION GTD_VCS_IDENTIFIER,
                          "copyright", copyright,
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "authors", authors,
@@ -158,8 +158,6 @@ gtd_application_show_about (GSimpleAction *simple,
                          "logo-icon-name", "org.gnome.Todo",
                          "translator-credits", _("translator-credits"),
                          NULL);
-  g_free (copyright);
-  g_date_time_unref (date);
 }
 
 static void
@@ -167,9 +165,9 @@ gtd_application_quit (GSimpleAction *simple,
                       GVariant      *parameter,
                       gpointer       user_data)
 {
-  GtdApplicationPrivate *priv = GTD_APPLICATION (user_data)->priv;
+  GtdApplication *self = GTD_APPLICATION (user_data);
 
-  gtk_widget_destroy (priv->window);
+  gtk_widget_destroy (self->window);
 }
 
 GtdApplication *
@@ -179,22 +177,16 @@ gtd_application_new (void)
 
   return g_object_new (GTD_TYPE_APPLICATION,
                        "application-id", "org.gnome.Todo",
-                       "flags", G_APPLICATION_FLAGS_NONE,
+                       "flags", G_APPLICATION_HANDLES_COMMAND_LINE,
                        "resource-base-path", "/org/gnome/todo",
                        NULL);
 }
 
 static void
-run_window (GtdApplication *application)
+run_window (GtdApplication *self)
 {
-  GtdApplicationPrivate *priv;
-
-  g_return_if_fail (GTD_IS_APPLICATION (application));
-
-  priv = application->priv;
-
-  gtk_widget_show (priv->window);
-  gtk_window_present (GTK_WINDOW (priv->window));
+  gtk_widget_show (self->window);
+  gtk_window_present (GTK_WINDOW (self->window));
 }
 
 /*
@@ -256,26 +248,24 @@ gtd_application_finalize (GObject *object)
 static void
 gtd_application_startup (GApplication *application)
 {
-  GtdApplicationPrivate *priv;
+  GtdApplication *self;
   g_autoptr (GtkCssProvider) css_provider;
   g_autoptr (GFile) css_file;
   g_autofree gchar *theme_name, *theme_uri;
 
-  priv = GTD_APPLICATION (application)->priv;
+  self = GTD_APPLICATION (application);
 
   /* add actions */
-  g_action_map_add_action_entries (G_ACTION_MAP (application),
+  g_action_map_add_action_entries (G_ACTION_MAP (self),
                                    gtd_application_entries,
                                    G_N_ELEMENTS (gtd_application_entries),
-                                   application);
+                                   self);
 
   G_APPLICATION_CLASS (gtd_application_parent_class)->startup (application);
 
-  /* manager */
-  priv->manager = gtd_manager_get_default ();
-
   /* window */
-  priv->window = gtd_window_new (GTD_APPLICATION (application));
+  gtk_window_set_default_icon_name ("org.gnome.Todo");
+  self->window = gtd_window_new (self);
 
   /* CSS provider */
   css_provider = gtk_css_provider_new ();
@@ -293,12 +283,31 @@ gtd_application_startup (GApplication *application)
     gtk_css_provider_load_from_resource (css_provider, "/org/gnome/todo/theme/Adwaita.css");
 
   /* plugin dialog */
-  priv->plugin_dialog = gtd_plugin_dialog_new ();
+  self->plugin_dialog = gtd_plugin_dialog_new ();
 
-  gtk_window_set_transient_for (GTK_WINDOW (priv->plugin_dialog), GTK_WINDOW (priv->window));
+  gtk_window_set_transient_for (GTK_WINDOW (self->plugin_dialog), GTK_WINDOW (self->window));
 
   /* Load the plugins */
-  gtd_manager_load_plugins (priv->manager);
+  gtd_manager_load_plugins (gtd_manager_get_default ());
+}
+
+static gint
+gtd_application_command_line (GApplication            *app,
+                              GApplicationCommandLine *command_line)
+{
+  GVariantDict *options;
+
+  options = g_application_command_line_get_options_dict (command_line);
+
+  if (g_variant_dict_contains (options, "quit"))
+    {
+      g_application_quit (app);
+      return 0;
+    }
+
+  g_application_activate (app);
+
+  return 0;
 }
 
 static gboolean
@@ -323,21 +332,12 @@ gtd_application_class_init (GtdApplicationClass *klass)
 
   application_class->activate = gtd_application_activate;
   application_class->startup = gtd_application_startup;
+  application_class->command_line = gtd_application_command_line;
   application_class->local_command_line = gtd_application_local_command_line;
 }
 
 static void
 gtd_application_init (GtdApplication *self)
 {
-  GtdApplicationPrivate *priv = gtd_application_get_instance_private (self);
-
-  self->priv = priv;
-}
-
-GtdManager*
-gtd_application_get_manager (GtdApplication *app)
-{
-  g_return_val_if_fail (GTD_IS_APPLICATION (app), NULL);
-
-  return app->priv->manager;
+  g_application_add_main_option_entries (G_APPLICATION (self), cmd_options);
 }
