@@ -16,7 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gtd-arrow-frame.h"
 #include "gtd-dnd-row.h"
 #include "gtd-edit-pane.h"
 #include "gtd-empty-list-widget.h"
@@ -64,10 +63,7 @@
 
 typedef struct
 {
-  GtdArrowFrame         *arrow_frame;
   GtkWidget             *dnd_row;
-  GtdEditPane           *edit_pane;
-  GtkRevealer           *edit_revealer;
   GtkWidget             *empty_box;
   GtkListBox            *listbox;
   GtkListBoxRow         *new_task_row;
@@ -584,17 +580,22 @@ gtd_task_list_view__update_empty_state (GtdTaskListView *view)
 
   for (l = tasks; l != NULL; l = l->next)
     {
-      if (!gtd_task_get_complete (l->data) ||
-          (priv->show_completed && gtd_task_get_complete (l->data)))
+      if (priv->show_completed || !gtd_task_get_complete (l->data))
         {
           is_empty = FALSE;
           break;
         }
     }
 
-  gtk_widget_set_visible (view->priv->empty_box, is_empty);
-  gtd_empty_list_widget_set_is_empty (GTD_EMPTY_LIST_WIDGET (view->priv->empty_box),
-                                      view->priv->complete_tasks == 0);
+  gtk_widget_set_visible (priv->empty_box, is_empty);
+  gtd_empty_list_widget_set_is_empty (GTD_EMPTY_LIST_WIDGET (priv->empty_box),
+                                      priv->complete_tasks == 0);
+
+  /*
+   * If there are visible tasks, we visually separate the New Task row by
+   * increasing the top margin to 12px.
+   */
+  gtk_widget_set_margin_top (GTK_WIDGET (priv->new_task_row), is_empty ? 3 : 12);
 
   g_list_free (tasks);
 }
@@ -635,19 +636,18 @@ ask_subtask_removal_warning (GtdTaskListView *self)
 }
 
 static void
-gtd_task_list_view__remove_task_cb (GtdEditPane *pane,
-                                    GtdTask     *task,
-                                    gpointer     user_data)
+remove_task_cb (GtdTaskRow      *row,
+                GtdTaskListView *self)
 {
   GtdTaskListViewPrivate *priv;
   GtdNotification *notification;
   RemoveTaskData *data;
   GtdWindow *window;
+  GtdTask *task;
   GList *subtasks;
   gchar *text;
 
-  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (user_data));
-
+  task = gtd_task_row_get_task (row);
   subtasks = gtd_task_get_subtasks (task);
 
   /*
@@ -658,26 +658,23 @@ gtd_task_list_view__remove_task_cb (GtdEditPane *pane,
     {
       gboolean should_remove_task;
 
-      should_remove_task = ask_subtask_removal_warning (user_data);
+      should_remove_task = ask_subtask_removal_warning (self);
 
       /* The user canceled the operation, do nothing */
       if (!should_remove_task)
         goto out;
     }
 
-  priv = GTD_TASK_LIST_VIEW (user_data)->priv;
+  priv = self->priv;
   text = g_strdup_printf (_("Task <b>%s</b> removed"), gtd_task_get_title (task));
-  window = GTD_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (user_data)));
+  window = GTD_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self)));
 
   data = g_new0 (RemoveTaskData, 1);
-  data->view = user_data;
+  data->view = self;
   data->task = task;
 
   /* Always remove tasks and subtasks */
-  iterate_subtasks (user_data,
-                    task,
-                    remove_task_from_list,
-                    FALSE);
+  iterate_subtasks (self, task, remove_task_from_list, FALSE);
 
   /*
    * Reset the DnD row, to avoid getting into an inconsistent state where
@@ -685,9 +682,6 @@ gtd_task_list_view__remove_task_cb (GtdEditPane *pane,
    */
   gtd_dnd_row_set_row_above (GTD_DND_ROW (priv->dnd_row), NULL);
 
-  /* Hide the edit panel */
-  gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
-
   /* Notify about the removal */
   notification = gtd_notification_new (text, 7500.0);
 
@@ -703,7 +697,7 @@ gtd_task_list_view__remove_task_cb (GtdEditPane *pane,
   gtd_window_notify (window, notification);
 
   /* Clear the active row */
-  set_active_row (user_data, NULL);
+  set_active_row (self, NULL);
 
   g_clear_pointer (&text, g_free);
 
@@ -711,31 +705,6 @@ out:
   g_clear_pointer (&subtasks, g_list_free);
 }
 
-static void
-gtd_task_list_view__edit_task_finished (GtdEditPane *pane,
-                                        GtdTask     *task,
-                                        gpointer     user_data)
-{
-  GtdTaskListViewPrivate *priv;
-
-  g_return_if_fail (GTD_IS_TASK (task));
-  g_return_if_fail (GTD_IS_EDIT_PANE (pane));
-  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (user_data));
-
-  priv = GTD_TASK_LIST_VIEW (user_data)->priv;
-
-  set_active_row (user_data, NULL);
-
-  gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
-
-  gtd_task_save (task);
-
-  gtd_manager_update_task (gtd_manager_get_default (), task);
-  real_save_task (GTD_TASK_LIST_VIEW (user_data), task);
-
-  gtk_list_box_invalidate_sort (priv->listbox);
-}
-
 static void
 gtd_task_list_view__color_changed (GtdTaskListView *self)
 {
@@ -835,31 +804,7 @@ static void
 task_row_entered_cb (GtdTaskListView *self,
                      GtdTaskRow      *row)
 {
-  GtdTaskListViewPrivate *priv = self->priv;
-  GtdTask *old_task;
-
-  old_task = gtd_edit_pane_get_task (priv->edit_pane);
-
-  /* Save the task previously edited */
-  if (old_task)
-    {
-      gtd_manager_update_task (gtd_manager_get_default (), old_task);
-      real_save_task (self, old_task);
-    }
-
   set_active_row (self, GTK_WIDGET (row));
-
-  /* If we focused the new task row, only activate it */
-  if (GTD_IS_NEW_TASK_ROW (row))
-    {
-      gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
-      return;
-    }
-
-  gtd_edit_pane_set_task (priv->edit_pane, gtd_task_row_get_task (row));
-
-  gtk_revealer_set_reveal_child (priv->edit_revealer, TRUE);
-  gtd_arrow_frame_set_row (priv->arrow_frame, row);
 }
 
 static void
@@ -867,21 +812,6 @@ task_row_exited_cb (GtdTaskListView *self,
                     GtdTaskRow      *row)
 {
   GtdTaskListViewPrivate *priv = self->priv;
-  GtdTask *old_task;
-
-  old_task = gtd_edit_pane_get_task (priv->edit_pane);
-
-  /* Save the task previously edited */
-  if (old_task)
-    {
-      gtd_manager_update_task (gtd_manager_get_default (), old_task);
-      real_save_task (self, old_task);
-    }
-
-  gtd_edit_pane_set_task (priv->edit_pane, NULL);
-
-  gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
-  gtd_arrow_frame_set_row (priv->arrow_frame, NULL);
 
   if (GTK_WIDGET (row) == priv->active_row &&
       priv->active_row != GTK_WIDGET (priv->new_task_row))
@@ -898,7 +828,11 @@ listbox_row_activated (GtkListBox      *listbox,
   if (!GTD_IS_TASK_ROW (row))
     return;
 
-  set_active_row (self, GTK_WIDGET (row));
+  /* Toggle the row */
+  if (gtd_task_row_get_active (GTD_TASK_ROW (row)))
+    set_active_row (self, NULL);
+  else
+    set_active_row (self, GTK_WIDGET (row));
 }
 
 static void
@@ -929,9 +863,12 @@ insert_task (GtdTaskListView *self,
                             G_CALLBACK (task_row_exited_cb),
                             self);
 
-  gtk_list_box_insert (priv->listbox,
-                       new_row,
-                       0);
+  g_signal_connect (new_row,
+                    "remove-task",
+                    G_CALLBACK (remove_task_cb),
+                    self);
+
+  gtk_list_box_insert (priv->listbox, new_row, 0);
 
   /*
    * Setup a sizegroup to let all the tasklist labels have
@@ -965,8 +902,6 @@ remove_task (GtdTaskListView *view,
   GList *children;
   GList *l;
 
-  gtd_arrow_frame_set_row (view->priv->arrow_frame, NULL);
-
   children = gtk_container_get_children (GTK_CONTAINER (view->priv->listbox));
 
   for (l = children; l != NULL; l = l->next)
@@ -990,7 +925,6 @@ remove_task (GtdTaskListView *view,
     }
 
   gtk_revealer_set_reveal_child (priv->revealer, FALSE);
-  gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
 
   g_list_free (children);
 }
@@ -1112,17 +1046,6 @@ task_completed_cb (GtdTask         *task,
   else
     priv->complete_tasks--;
 
-  /*
-   * If we're editing the task and it get completed, hide the edit
-   * pane and the task.
-   */
-  if (task_complete &&
-      task == gtd_edit_pane_get_task (priv->edit_pane))
-    {
-      gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
-      gtd_edit_pane_set_task (priv->edit_pane, NULL);
-    }
-
   if (!priv->show_completed)
     {
       IterateSubtaskFunc func;
@@ -1158,10 +1081,10 @@ gtd_task_list_view__task_added (GtdTaskList     *list,
 }
 
 static void
-gtd_task_list_view__create_task (GtdTaskRow  *row,
-                                 GtdTask     *task,
-                                 GtdTaskList *list,
-                                 gpointer     user_data)
+create_task_cb (GtdTaskRow  *row,
+                GtdTask     *task,
+                GtdTaskList *list,
+                gpointer     user_data)
 {
   GtdTaskListViewPrivate *priv;
 
@@ -1410,13 +1333,33 @@ listbox_drag_motion (GtkListBox      *listbox,
 {
   GtdTaskListViewPrivate *priv;
   GtkListBoxRow *hovered_row;
+  GtkListBoxRow *source_row;
   GtkListBoxRow *task_row;
   GtkListBoxRow *row_above_dnd;
+  GtkWidget *source_widget;
+  gboolean success;
   gint row_x, row_y, row_height;
 
   priv = gtd_task_list_view_get_instance_private (self);
+  source_widget = gtk_drag_get_source_widget (context);
+  source_row = GTK_LIST_BOX_ROW (gtk_widget_get_ancestor (source_widget, GTK_TYPE_LIST_BOX_ROW));
   hovered_row = gtk_list_box_get_row_at_y (listbox, y);
 
+  /* Update the x value according to the current offset */
+  if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
+    x += gtd_task_row_get_x_offset (GTD_TASK_ROW (source_row));
+  else
+    x -= gtd_task_row_get_x_offset (GTD_TASK_ROW (source_row));
+
+  /* Make sure the DnD row always have the same height of the dragged row */
+  gtk_widget_set_size_request (priv->dnd_row,
+                               -1,
+                               gtk_widget_get_allocated_height (GTK_WIDGET (source_row)));
+
+  gtk_widget_queue_resize (priv->dnd_row);
+
+  gdk_drag_status (context, GDK_ACTION_MOVE, time);
+
   /*
    * When not hovering any row, we still have to make sure that the listbox is a valid
    * drop target. Otherwise, the user can drop at the space after the rows, and the row
@@ -1484,12 +1427,9 @@ listbox_drag_motion (GtkListBox      *listbox,
   /* Check if we're not trying to add a subtask */
   if (row_above_dnd)
     {
-      GtkWidget *dnd_widget, *dnd_row;
       GtdTask *row_above_task, *dnd_task;
 
-      dnd_widget = gtk_drag_get_source_widget (context);
-      dnd_row = gtk_widget_get_ancestor (dnd_widget, GTK_TYPE_LIST_BOX_ROW);
-      dnd_task = gtd_task_row_get_task (GTD_TASK_ROW (dnd_row));
+      dnd_task = gtd_task_row_get_task (GTD_TASK_ROW (source_row));
       row_above_task = gtd_task_row_get_task (GTD_TASK_ROW (row_above_dnd));
 
       /* Forbid DnD'ing a row into a subtask */
@@ -1510,17 +1450,11 @@ success:
    * Also pass the current motion to the DnD row, so it correctly
    * adjusts itself - even when the DnD is hovering another row.
    */
-  gtd_dnd_row_drag_motion (GTK_WIDGET (priv->dnd_row),
-                           context,
-                           x,
-                           y,
-                           time);
+  success = gtd_dnd_row_drag_motion (GTK_WIDGET (priv->dnd_row), context, x, y, time);
 
   check_dnd_scroll (self, FALSE, y);
 
-  gdk_drag_status (context, GDK_ACTION_COPY, time);
-
-  return TRUE;
+  return success;
 
 fail:
   return FALSE;
@@ -1535,18 +1469,16 @@ listbox_drag_drop (GtkWidget       *widget,
                    GtdTaskListView *self)
 {
   GtdTaskListViewPrivate *priv;
+  gboolean success;
 
   priv = gtd_task_list_view_get_instance_private (self);
-
-  gtd_dnd_row_drag_drop (GTK_WIDGET (priv->dnd_row),
-                         context,
-                         x,
-                         y,
-                         time);
+  success = gtd_dnd_row_drag_drop (GTK_WIDGET (priv->dnd_row), context, x, y, time);
 
   check_dnd_scroll (self, TRUE, -1);
 
-  return TRUE;
+  gtk_drag_finish (context, success, TRUE, time);
+
+  return success;
 }
 
 static void
@@ -1584,7 +1516,11 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
 
   widget_class->map = gtd_task_list_view_map;
 
+  g_type_ensure (GTD_TYPE_EDIT_PANE);
+  g_type_ensure (GTD_TYPE_NEW_TASK_ROW);
   g_type_ensure (GTD_TYPE_TASK_ROW);
+  g_type_ensure (GTD_TYPE_DND_ROW);
+  g_type_ensure (GTD_TYPE_EMPTY_LIST_WIDGET);
 
   /**
    * GtdTaskListView::color:
@@ -1673,11 +1609,8 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/todo/ui/list-view.ui");
 
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, arrow_frame);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, dnd_row);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, due_date_sizegroup);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, edit_pane);
-  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, edit_revealer);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, empty_box);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, listbox);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, revealer);
@@ -1688,10 +1621,8 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, viewport);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, scrolled_window);
 
-  gtk_widget_class_bind_template_callback (widget_class, gtd_task_list_view__create_task);
+  gtk_widget_class_bind_template_callback (widget_class, create_task_cb);
   gtk_widget_class_bind_template_callback (widget_class, gtd_task_list_view__done_button_clicked);
-  gtk_widget_class_bind_template_callback (widget_class, gtd_task_list_view__edit_task_finished);
-  gtk_widget_class_bind_template_callback (widget_class, gtd_task_list_view__remove_task_cb);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_drop);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_leave);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_motion);
@@ -1699,7 +1630,7 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, task_row_entered_cb);
   gtk_widget_class_bind_template_callback (widget_class, task_row_exited_cb);
 
-  gtk_widget_class_set_css_name (widget_class, "task-list-view");
+  gtk_widget_class_set_css_name (widget_class, "tasklistview");
 }
 
 static void
@@ -1718,7 +1649,7 @@ gtd_task_list_view_init (GtdTaskListView *self)
                      0,
                      NULL,
                      0,
-                     GDK_ACTION_COPY);
+                     GDK_ACTION_MOVE);
 }
 
 /**
@@ -1906,7 +1837,6 @@ gtd_task_list_view_set_task_list (GtdTaskListView *view,
 
   if (!list)
     {
-      gtd_edit_pane_set_task (GTD_EDIT_PANE (priv->edit_pane), NULL);
       gtd_task_list_view_set_list (view, NULL);
       return;
     }
@@ -1934,7 +1864,6 @@ gtd_task_list_view_set_task_list (GtdTaskListView *view,
   task_list = gtd_task_list_get_tasks (list);
 
   gtd_task_list_view_set_list (view, task_list);
-  gtd_edit_pane_set_task (priv->edit_pane, NULL);
 
   g_list_free (task_list);
 
