@@ -21,7 +21,6 @@
 #include "gtd-task-list.h"
 
 #include <glib/gi18n.h>
-#include <libecal/libecal.h>
 
 /**
  * SECTION:gtd-task-list
@@ -34,12 +33,6 @@
  * a name and the provider who generated it.
  */
 
-typedef struct
-{
-  gchar   *parent_uid;
-  GtdTask *child;
-} PendingSubtaskData;
-
 typedef struct
 {
   GList               *tasks;
@@ -47,7 +40,6 @@ typedef struct
   GdkRGBA             *color;
 
   GHashTable          *uid_to_task;
-  GPtrArray           *pending_subtasks;
 
   gchar               *name;
   gboolean             removable : 1;
@@ -75,91 +67,6 @@ enum
   LAST_PROP
 };
 
-static PendingSubtaskData*
-pending_subtask_data_new (GtdTaskList *self,
-                          GtdTask     *child,
-                          const gchar *parent_uid)
-{
-  PendingSubtaskData *data;
-
-  data = g_new0 (PendingSubtaskData, 1);
-  data->child = child;
-  data->parent_uid = g_strdup (parent_uid);
-
-  return data;
-}
-
-static void
-pending_subtask_data_free (PendingSubtaskData *data)
-{
-  g_free (data->parent_uid);
-  g_free (data);
-}
-
-static void
-setup_parent_task (GtdTaskList *self,
-                   GtdTask     *task)
-{
-  GtdTaskListPrivate *priv;
-  ECalComponent *component;
-  icalcomponent *ical_comp;
-  icalproperty *property;
-  GtdTask *parent_task;
-  const gchar *parent_uid;
-
-  priv = gtd_task_list_get_instance_private (self);
-  component = gtd_task_get_component (task);
-  ical_comp = e_cal_component_get_icalcomponent (component);
-  property = icalcomponent_get_first_property (ical_comp, ICAL_RELATEDTO_PROPERTY);
-
-  if (!property)
-    return;
-
-  parent_uid = icalproperty_get_relatedto (property);
-  parent_task = g_hash_table_lookup (priv->uid_to_task, parent_uid);
-
-  if (parent_task)
-    {
-      gtd_task_add_subtask (parent_task, task);
-    }
-  else
-    {
-      PendingSubtaskData *data;
-
-      data = pending_subtask_data_new (self, task, parent_uid);
-
-      g_ptr_array_add (priv->pending_subtasks, data);
-    }
-}
-
-static void
-process_pending_subtasks (GtdTaskList *self,
-                          GtdTask     *task)
-{
-  GtdTaskListPrivate *priv;
-  ECalComponentId *id;
-  guint i;
-
-  priv = gtd_task_list_get_instance_private (self);
-  id = e_cal_component_get_id (gtd_task_get_component (task));
-
-  for (i = 0; i < priv->pending_subtasks->len; i++)
-    {
-      PendingSubtaskData *data;
-
-      data = g_ptr_array_index (priv->pending_subtasks, i);
-
-      if (g_strcmp0 (id->uid, data->parent_uid) == 0)
-        {
-          gtd_task_add_subtask (task, data->child);
-          g_ptr_array_remove (priv->pending_subtasks, data);
-          i--;
-        }
-    }
-
-  e_cal_component_free_id (id);
-}
-
 static void
 task_changed_cb (GtdTask     *task,
                  GParamSpec  *pspec,
@@ -174,7 +81,6 @@ gtd_task_list_finalize (GObject *object)
   GtdTaskList *self = (GtdTaskList*) object;
   GtdTaskListPrivate *priv = gtd_task_list_get_instance_private (self);
 
-  g_clear_object (&priv->pending_subtasks);
   g_clear_object (&priv->provider);
 
   g_clear_pointer (&priv->uid_to_task, g_hash_table_destroy);
@@ -385,8 +291,6 @@ gtd_task_list_init (GtdTaskList *self)
                                              g_str_equal,
                                              g_free,
                                              NULL);
-
-  priv->pending_subtasks = g_ptr_array_new_with_free_func ((GDestroyNotify) pending_subtask_data_free);
 }
 
 /**
@@ -595,15 +499,13 @@ gtd_task_list_save_task (GtdTaskList *list,
     }
   else
     {
-      ECalComponentId *id;
+      const gchar *uid;
 
-      id = e_cal_component_get_id (gtd_task_get_component (task));
+      uid = gtd_object_get_uid (GTD_OBJECT (task));
 
-      priv->tasks = g_list_append (priv->tasks, task);
+      priv->tasks = g_list_prepend (priv->tasks, task);
 
-      g_hash_table_insert (priv->uid_to_task, g_strdup (id->uid), task);
-      process_pending_subtasks (list, task);
-      setup_parent_task (list, task);
+      g_hash_table_insert (priv->uid_to_task, g_strdup (uid), task);
 
       g_signal_connect (task,
                         "notify",
@@ -611,8 +513,6 @@ gtd_task_list_save_task (GtdTaskList *list,
                         list);
 
       g_signal_emit (list, signals[TASK_ADDED], 0, task);
-
-      e_cal_component_free_id (id);
     }
 }
 
@@ -715,3 +615,25 @@ gtd_task_list_set_is_removable (GtdTaskList *list,
       g_object_notify (G_OBJECT (list), "is-removable");
     }
 }
+
+/**
+ * gtd_task_list_get_task_by_id:
+ * @list: a #GtdTaskList
+ * @id: the id of the task
+ *
+ * Retrieves a task from @self with the given @id.
+ *
+ * Returns: (transfer none)(nullable): a #GtdTask, or %NULL
+ */
+GtdTask*
+gtd_task_list_get_task_by_id (GtdTaskList *self,
+                              const gchar *id)
+{
+  GtdTaskListPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_TASK_LIST (self), NULL);
+
+  priv = gtd_task_list_get_instance_private (self);
+
+  return g_hash_table_lookup (priv->uid_to_task, id);
+}
