@@ -16,85 +16,100 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "gtd-edit-pane.h"
+#include "gtd-expandable-entry.h"
+#include "gtd-manager.h"
 #include "gtd-provider.h"
+#include "gtd-rows-common-private.h"
 #include "gtd-task-row.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
+#include "gtd-task-list-view.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 struct _GtdTaskRow
 {
-  GtkListBoxRow      parent;
+  GtkListBoxRow       parent;
 
   /*<private>*/
-  GtkRevealer               *revealer;
-  GtkStack                  *stack;
+  GtkRevealer        *revealer;
 
-  /* new task widgets */
-  GtkStack                  *done_check;
-  GtkEntry                  *new_task_entry;
-  GtkStack                  *new_task_stack;
+  GtkWidget          *done_check;
+  GtkWidget          *edit_panel;
+  GtkWidget          *edit_panel_revealer;
+  GtkWidget          *header_event_box;
+  GtkWidget          *title_entry;
+  GtkWidget          *toggle_button;
 
   /* task widgets */
-  GtkEntry                  *title_entry;
-  GtkLabel                  *task_date_label;
-  GtkLabel                  *task_list_label;
-  GtkStack                  *task_stack;
-  GtkSpinner                *task_loading_spinner;
-  GtkLabel                  *title_label;
+  GtkLabel           *task_date_label;
+  GtkLabel           *task_list_label;
 
   /* dnd widgets */
-  GtkWidget                 *dnd_box;
-  GtkWidget                 *dnd_event_box;
-  GtkWidget                 *dnd_icon;
-  gdouble                    clicked_x;
-  gdouble                    clicked_y;
+  GtkWidget          *dnd_box;
+  GtkWidget          *dnd_event_box;
+  GtkWidget          *dnd_icon;
+  gint                clicked_x;
+  gint                clicked_y;
 
-  gboolean                   handle_subtasks : 1;
+  gboolean            handle_subtasks : 1;
 
   /* data */
-  gboolean                   new_task_mode;
-  GtdTask                   *task;
+  GtdTask            *task;
 
-  gint                       destroy_row_timeout_id;
-
-  gboolean                   active;
+  gint                destroy_row_timeout_id;
+  gboolean            active;
 };
 
-#define PRIORITY_ICON_SIZE         8
-#define DND_ICON_SCALE             0.85
+#define PRIORITY_ICON_SIZE 8
 
 G_DEFINE_TYPE (GtdTaskRow, gtd_task_row, GTK_TYPE_LIST_BOX_ROW)
 
-enum {
+enum
+{
   ENTER,
   EXIT,
-  CREATE_TASK,
+  REMOVE_TASK,
   NUM_SIGNALS
 };
 
-enum {
+enum
+{
   PROP_0,
   PROP_HANDLE_SUBTASKS,
-  PROP_NEW_TASK_MODE,
   PROP_TASK,
   LAST_PROP
 };
 
-typedef enum
-{
-  CURSOR_NONE,
-  CURSOR_GRAB,
-  CURSOR_GRABBING
-} CursorType;
-
 static guint signals[NUM_SIGNALS] = { 0, };
 
+/*
+ * Auxiliary methods
+ */
+
+static GtkWidget*
+create_transient_row (GtdTaskRow *self)
+{
+  GtdTaskRow *new_row;
+
+  new_row = GTD_TASK_ROW (gtd_task_row_new (self->task));
+  gtk_revealer_set_transition_duration (new_row->revealer, 0);
+  gtk_revealer_set_reveal_child (new_row->revealer, TRUE);
+
+  gtk_widget_set_size_request (GTK_WIDGET (new_row),
+                               gtk_widget_get_allocated_width (GTK_WIDGET (self)),
+                               -1);
+
+  gtk_revealer_set_reveal_child (GTK_REVEALER (new_row->edit_panel_revealer), self->active);
+
+  return GTK_WIDGET (new_row);
+}
+
 static void
-set_dnd_cursor (GtkWidget  *widget,
-                CursorType  type)
+set_cursor (GtkWidget   *widget,
+            const gchar *cursor_name)
 {
   GdkDisplay *display;
   GdkCursor *cursor;
@@ -102,25 +117,11 @@ set_dnd_cursor (GtkWidget  *widget,
   if (!gtk_widget_get_realized (widget))
     return;
 
+  cursor = NULL;
   display = gtk_widget_get_display (widget);
 
-  switch (type)
-    {
-    case CURSOR_NONE:
-      cursor = NULL;
-      break;
-
-    case CURSOR_GRAB:
-      cursor = gdk_cursor_new_from_name (display, "grab");
-      break;
-
-    case CURSOR_GRABBING:
-      cursor = gdk_cursor_new_from_name (display, "grabbing");
-      break;
-
-    default:
-      cursor = NULL;
-    }
+  if (cursor_name)
+    cursor = gdk_cursor_new_from_name (display, cursor_name);
 
   gdk_window_set_cursor (gtk_widget_get_window (widget), cursor);
   gdk_display_flush (display);
@@ -128,55 +129,66 @@ set_dnd_cursor (GtkWidget  *widget,
   g_clear_object (&cursor);
 }
 
-static cairo_surface_t*
-get_dnd_icon (GtdTaskRow *self)
+
+/*
+ * Callbacks
+ */
+
+static void
+toggle_active_cb (GtkWidget  *button,
+                  GtdTaskRow *self)
 {
-  cairo_surface_t *surface;
-  GtkWidget *widget;
-  cairo_t *cr;
-  gint real_x;
+  if (self->active)
+    gtd_manager_update_task (gtd_manager_get_default (), self->task);
 
-  widget = GTK_WIDGET (self);
-  gtk_widget_translate_coordinates (self->dnd_event_box,
-                                    gtk_widget_get_parent (widget),
-                                    0,
-                                    0,
-                                    &real_x,
-                                    NULL);
+  gtd_task_row_set_active (self, !self->active);
+}
 
-  /* Make it transparent */
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                        gtk_widget_get_allocated_width (widget) * DND_ICON_SCALE,
-                                        gtk_widget_get_allocated_height (widget) * DND_ICON_SCALE);
-
-  cr = cairo_create (surface);
-  cairo_scale (cr, DND_ICON_SCALE, DND_ICON_SCALE);
-
-  gtk_widget_draw (widget, cr);
-
-  cairo_surface_set_device_offset (surface,
-                                   -(self->clicked_x + real_x) * DND_ICON_SCALE,
-                                   -self->clicked_y * DND_ICON_SCALE);
-
-  return surface;
+static void
+remove_task_cb (GtdEditPane *edit_panel,
+                GtdTask     *task,
+                GtdTaskRow  *self)
+{
+  g_signal_emit (self, signals[REMOVE_TASK], 0);
 }
 
 static gboolean
-mouse_out_event (GtkWidget  *widget,
-                 GdkEvent   *event,
-                 GtdTaskRow *self)
+mouse_out_event_cb (GtkWidget  *widget,
+                    GdkEvent   *event,
+                    GtdTaskRow *self)
 {
-  set_dnd_cursor (widget, CURSOR_NONE);
+  set_cursor (widget, NULL);
 
   return GDK_EVENT_STOP;
 }
 
 static gboolean
-mouse_over_event (GtkWidget  *widget,
-                  GdkEvent   *event,
-                  GtdTaskRow *self)
+mouse_over_event_cb (GtkWidget  *widget,
+                     GdkEvent   *event,
+                     GtdTaskRow *self)
 {
-  set_dnd_cursor (widget, CURSOR_GRAB);
+  set_cursor (widget, "pointer");
+
+  return GDK_EVENT_STOP;
+}
+
+
+static gboolean
+mouse_out_dnd_event_cb (GtkWidget  *widget,
+                        GdkEvent   *event,
+                        GtdTaskRow *self)
+{
+  set_cursor (widget, NULL);
+
+  return GDK_EVENT_STOP;
+}
+
+static gboolean
+mouse_over_dnd_event_cb (GtkWidget  *widget,
+                         GdkEvent   *event,
+                         GtdTaskRow *self)
+{
+  set_cursor (widget, "grab");
 
   return GDK_EVENT_STOP;
 }
@@ -186,40 +198,82 @@ button_press_event (GtkWidget      *widget,
                     GdkEventButton *event,
                     GtdTaskRow     *self)
 {
-  self->clicked_x = event->x;
-  self->clicked_y = event->y;
+  gint real_x;
+  gint real_y;
+
+  gtk_widget_translate_coordinates (widget,
+                                    GTK_WIDGET (self),
+                                    event->x,
+                                    event->y,
+                                    &real_x,
+                                    &real_y);
+
+  self->clicked_x = real_x;
+  self->clicked_y = real_y;
 
   return GDK_EVENT_PROPAGATE;
 }
 
 static void
-drag_begin_cb (GtkWidget      *widget,
+drag_begin_cb (GtkWidget      *event_box,
                GdkDragContext *context,
                GtdTaskRow     *self)
 {
-  cairo_surface_t *surface;
+  GtkWidget *widget, *new_row;
+  gint x_offset;
 
-  surface = get_dnd_icon (self);
+  widget = GTK_WIDGET (self);
 
-  set_dnd_cursor (widget, CURSOR_GRABBING);
+  set_cursor (widget, "grabbing");
 
-  gtk_drag_set_icon_surface (context, surface);
+  /*
+   * gtk_drag_set_icon_widget() inserts the row in a different GtkWindow, so
+   * we have to create a new, transient row.
+   */
+  new_row = create_transient_row (self);
 
-  gtk_widget_hide (GTK_WIDGET (self));
+  if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
+    x_offset = gtk_widget_get_margin_end (GTK_WIDGET (self));
+  else
+    x_offset = gtk_widget_get_margin_start (GTK_WIDGET (self));
 
-  g_clear_pointer (&surface, cairo_surface_destroy);
+  gtk_drag_set_icon_widget (context,
+                            new_row,
+                            self->clicked_x + x_offset,
+                            self->clicked_y);
+
+  gtk_widget_hide (widget);
 }
 
 static void
-gtd_task_row__priority_changed_cb (GtdTaskRow *row,
-                                   GParamSpec *spec,
-                                   GObject    *object)
+drag_end_cb (GtkWidget      *event_box,
+             GdkDragContext *context,
+             GtdTaskRow     *self)
+{
+  set_cursor (GTK_WIDGET (self), NULL);
+  gtk_widget_show (GTK_WIDGET (self));
+}
+
+static gboolean
+drag_failed_cb (GtkWidget      *widget,
+                GdkDragContext *context,
+                GtkDragResult   result,
+                GtdTaskRow     *self)
+{
+  set_cursor (GTK_WIDGET (self), NULL);
+  gtk_widget_show (GTK_WIDGET (self));
+
+  return FALSE;
+}
+
+static void
+priority_changed_cb (GtdTaskRow *row,
+                     GParamSpec *spec,
+                     GObject    *object)
 {
   GtkStyleContext *context;
   gint priority;
 
-  g_return_if_fail (GTD_IS_TASK_ROW (row));
-
   context = gtk_widget_get_style_context (GTK_WIDGET (row));
   priority = gtd_task_get_priority (GTD_TASK (object));
 
@@ -265,13 +319,52 @@ complete_changed_cb (GtdTaskRow *self,
     gtk_style_context_remove_class (context, "complete");
 }
 
+static void
+toggle_complete_cb (GtkRevealer *revealer,
+                    GParamSpec  *pspec,
+                    GtdTaskRow  *self)
+{
+  g_signal_handlers_disconnect_by_func (revealer, toggle_complete_cb, self);
+
+  gtd_task_set_complete (self->task, !gtd_task_get_complete (self->task));
+}
+
+static void
+complete_check_toggled_cb (GtkToggleButton *button,
+                           GtdTaskRow      *self)
+{
+  GtdTaskListView *listview;
+
+  listview = GTD_TASK_LIST_VIEW (gtk_widget_get_ancestor (GTK_WIDGET (self),
+                                                          GTD_TYPE_TASK_LIST_VIEW));
+
+  /*
+   * If the parent list view is showing completed tasks, we
+   * don't have to hide the row. Simply toggle the 'complete'
+   * property of the task.
+   */
+  if (gtd_task_list_view_get_show_completed (listview))
+    {
+      gtd_task_set_complete (self->task, !gtd_task_get_complete (self->task));
+      return;
+    }
+
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer),
+                                 !gtk_toggle_button_get_active (button));
+
+  g_signal_connect (self->revealer,
+                    "notify::child-revealed",
+                    G_CALLBACK (toggle_complete_cb),
+                    self);
+}
+
 static void
 depth_changed_cb (GtdTaskRow *self,
                   GParamSpec *pspec,
                   GtdTask    *task)
 {
-  gtk_widget_set_margin_start (self->dnd_box,
-                               self->handle_subtasks ? 32 * gtd_task_get_depth (task) : 0);
+  gtk_widget_set_margin_start (GTK_WIDGET (self),
+                               self->handle_subtasks ? 32 * gtd_task_get_depth (task) + 3: 3);
 }
 
 static gboolean
@@ -280,8 +373,8 @@ gtd_task_row__date_changed_binding (GBinding     *binding,
                                     GValue       *to_value,
                                     gpointer      user_data)
 {
+  g_autofree gchar *new_label = NULL;
   GDateTime *dt;
-  gchar *new_label = NULL;
 
   g_return_val_if_fail (GTD_IS_TASK_ROW (user_data), FALSE);
 
@@ -289,7 +382,7 @@ gtd_task_row__date_changed_binding (GBinding     *binding,
 
   if (dt)
     {
-      GDateTime *today = g_date_time_new_now_local ();
+      g_autoptr (GDateTime) today = g_date_time_new_now_local ();
 
       if (g_date_time_get_year (dt) == g_date_time_get_year (today) &&
           g_date_time_get_month (dt) == g_date_time_get_month (today))
@@ -320,33 +413,17 @@ gtd_task_row__date_changed_binding (GBinding     *binding,
         {
           new_label = g_date_time_format (dt, "%x");
         }
-
-      g_date_time_unref (today);
-
     }
   else
     {
-      new_label = g_strdup (_("No date set"));
+      new_label = g_strdup ("");
     }
 
   g_value_set_string (to_value, new_label);
 
-  g_free (new_label);
-
   return TRUE;
 }
 
-static GtdTask*
-gtd_task_row__create_task_for_name (const gchar *name)
-{
-  GtdTask *task = gtd_task_new (NULL);
-
-  gtd_task_set_title (task, name);
-  gtd_task_save (task);
-
-  return task;
-}
-
 static gboolean
 gtd_task_row__destroy_cb (GtkWidget *row)
 {
@@ -355,13 +432,10 @@ gtd_task_row__destroy_cb (GtkWidget *row)
   return G_SOURCE_REMOVE;
 }
 
-GtkWidget*
-gtd_task_row_new (GtdTask *task)
-{
-  return g_object_new (GTD_TYPE_TASK_ROW,
-                       "task", task,
-                       NULL);
-}
+
+/*
+ * GtkWidget overrides
+ */
 
 static gboolean
 gtd_task_row__key_press_event (GtkWidget   *row,
@@ -373,55 +447,16 @@ gtd_task_row__key_press_event (GtkWidget   *row,
       !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK))) // No modifiers together
     {
       self->active = FALSE;
-
-      if (self->new_task_mode)
-        {
-          gtk_stack_set_visible_child_name (self->new_task_stack, "label");
-          gtk_entry_set_text (self->new_task_entry, "");
-          return TRUE;
-        }
-      else
-        {
-          g_signal_emit (row, signals[EXIT], 0);
-        }
+      g_signal_emit (row, signals[EXIT], 0);
     }
 
   return FALSE;
 }
 
-static gboolean
-gtd_task_row_focus_in_event (GtkWidget     *widget,
-                             GdkEventFocus *event)
-{
-  gtd_task_row_set_active (GTD_TASK_ROW (widget), TRUE);
 
-  return GDK_EVENT_PROPAGATE;
-}
-
-static void
-gtd_task_row__entry_activated (GtkEntry *entry,
-                               gpointer  user_data)
-{
-  GtdTaskRow *self = GTD_TASK_ROW (user_data);
-
-  g_return_if_fail (GTD_IS_TASK_ROW (user_data));
-  g_return_if_fail (GTK_IS_ENTRY (entry));
-
-  if (entry == self->new_task_entry)
-    {
-      GtdTask *new_task;
-
-      /* Cannot create empty tasks */
-      if (gtk_entry_get_text_length (self->new_task_entry) == 0)
-        return;
-
-      new_task = gtd_task_row__create_task_for_name (gtk_entry_get_text (self->new_task_entry));
-
-      g_signal_emit (user_data, signals[CREATE_TASK], 0, new_task);
-
-      gtk_entry_set_text (self->new_task_entry, "");
-    }
-}
+/*
+ * GObject overrides
+ */
 
 static void
 gtd_task_row_finalize (GObject *object)
@@ -444,17 +479,10 @@ gtd_task_row_dispose (GObject *object)
 
   if (task)
     {
-      g_signal_handlers_disconnect_by_func (task,
-                                        depth_changed_cb,
-                                        self);
+      g_signal_handlers_disconnect_by_func (task, depth_changed_cb, self);
+      g_signal_handlers_disconnect_by_func (task, complete_changed_cb, self);
 
-      g_signal_handlers_disconnect_by_func (task,
-                                            complete_changed_cb,
-                                            self);
-
-      g_signal_handlers_disconnect_by_func (task,
-                                            gtd_task_row__priority_changed_cb,
-                                            self);
+      g_signal_handlers_disconnect_by_func (task, priority_changed_cb, self);
     }
 
   G_OBJECT_CLASS (gtd_task_row_parent_class)->dispose (object);
@@ -474,10 +502,6 @@ gtd_task_row_get_property (GObject    *object,
       g_value_set_boolean (value, self->handle_subtasks);
       break;
 
-    case PROP_NEW_TASK_MODE:
-      g_value_set_boolean (value, self->new_task_mode);
-      break;
-
     case PROP_TASK:
       g_value_set_object (value, self->task);
       break;
@@ -501,10 +525,6 @@ gtd_task_row_set_property (GObject      *object,
       gtd_task_row_set_handle_subtasks (self, g_value_get_boolean (value));
       break;
 
-    case PROP_NEW_TASK_MODE:
-      gtd_task_row_set_new_task_mode (self, g_value_get_boolean (value));
-      break;
-
     case PROP_TASK:
       gtd_task_row_set_task (self, g_value_get_object (value));
       break;
@@ -517,16 +537,18 @@ gtd_task_row_set_property (GObject      *object,
 static void
 gtd_task_row_class_init (GtdTaskRowClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->dispose = gtd_task_row_dispose;
   object_class->finalize = gtd_task_row_finalize;
   object_class->get_property = gtd_task_row_get_property;
   object_class->set_property = gtd_task_row_set_property;
 
-  widget_class->focus_in_event = gtd_task_row_focus_in_event;
   widget_class->key_press_event = gtd_task_row__key_press_event;
+  widget_class->get_preferred_width = gtd_row_get_preferred_width_with_max;
+
+  g_type_ensure (GTD_TYPE_EXPANDABLE_ENTRY);
 
   /**
    * GtdTaskRow::handle-subtasks:
@@ -542,20 +564,6 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
                                 TRUE,
                                 G_PARAM_READWRITE));
 
-  /**
-   * GtdTaskRow::new-task-mode:
-   *
-   * If the row is used to add new tasks.
-   */
-  g_object_class_install_property (
-          object_class,
-          PROP_NEW_TASK_MODE,
-          g_param_spec_boolean ("new-task-mode",
-                                "If the row is used to add a new task",
-                                "Whether the row is used to add a new task",
-                                FALSE,
-                                G_PARAM_READWRITE));
-
   /**
    * GtdTaskRow::task:
    *
@@ -601,11 +609,11 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
                                 0);
 
   /**
-   * GtdTaskRow::create-task:
+   * GtdTaskRow::remove-task:
    *
-   * Emitted when the row wants the parent widget to create a new task.
+   * Emitted when the user wants to delete the task represented by this row.
    */
-  signals[CREATE_TASK] = g_signal_new ("create-task",
+  signals[REMOVE_TASK] = g_signal_new ("remove-task",
                                        GTD_TYPE_TASK_ROW,
                                        G_SIGNAL_RUN_LAST,
                                        0,
@@ -613,8 +621,7 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
                                        NULL,
                                        NULL,
                                        G_TYPE_NONE,
-                                       1,
-                                       GTD_TYPE_TASK);
+                                       0);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/todo/ui/task-row.ui");
 
@@ -622,22 +629,26 @@ gtd_task_row_class_init (GtdTaskRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_event_box);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, dnd_icon);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, done_check);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, stack);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, new_task_entry);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, new_task_stack);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, edit_panel);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, edit_panel_revealer);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, header_event_box);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, revealer);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_date_label);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_list_label);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_stack);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, task_loading_spinner);
   gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, title_entry);
-  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, title_label);
+  gtk_widget_class_bind_template_child (widget_class, GtdTaskRow, toggle_button);
 
   gtk_widget_class_bind_template_callback (widget_class, button_press_event);
+  gtk_widget_class_bind_template_callback (widget_class, complete_check_toggled_cb);
   gtk_widget_class_bind_template_callback (widget_class, drag_begin_cb);
-  gtk_widget_class_bind_template_callback (widget_class, mouse_out_event);
-  gtk_widget_class_bind_template_callback (widget_class, mouse_over_event);
-  gtk_widget_class_bind_template_callback (widget_class, gtd_task_row__entry_activated);
+  gtk_widget_class_bind_template_callback (widget_class, drag_end_cb);
+  gtk_widget_class_bind_template_callback (widget_class, drag_failed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_out_event_cb);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_out_dnd_event_cb);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_over_event_cb);
+  gtk_widget_class_bind_template_callback (widget_class, mouse_over_dnd_event_cb);
+  gtk_widget_class_bind_template_callback (widget_class, remove_task_cb);
+  gtk_widget_class_bind_template_callback (widget_class, toggle_active_cb);
 
   gtk_widget_class_set_css_name (widget_class, "taskrow");
 }
@@ -646,6 +657,7 @@ static void
 gtd_task_row_init (GtdTaskRow *self)
 {
   self->handle_subtasks = TRUE;
+  self->active = FALSE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -654,55 +666,22 @@ gtd_task_row_init (GtdTaskRow *self)
                        GDK_BUTTON1_MASK,
                        NULL,
                        0,
-                       GDK_ACTION_COPY);
+                       GDK_ACTION_MOVE);
+
+  /* But the rest of the row header is also draggable */
+  gtk_drag_source_set (self->header_event_box,
+                       GDK_BUTTON1_MASK,
+                       NULL,
+                       0,
+                       GDK_ACTION_MOVE);
 }
 
-/**
- * gtd_task_row_get_new_task_mode:
- * @row: a #GtdTaskRow
- *
- * Whether @row is in new task mode.
- *
- * Returns: %TRUE if @row is in new task mode, %FALSE otherwise
- */
-gboolean
-gtd_task_row_get_new_task_mode (GtdTaskRow *row)
+GtkWidget*
+gtd_task_row_new (GtdTask *task)
 {
-  g_return_val_if_fail (GTD_IS_TASK_ROW (row), FALSE);
-
-  return row->new_task_mode;
-}
-
-/**
- * gtd_task_row_set_new_task_mode:
- * @row: a #GtdTaskRow
- * @new_task_mode: %TRUE to set new task mode, %FALSE otherwise
- *
- * Sets @row new task mode to @new_task_mode. It is up to the caller
- * to set GtdTaskRow::task to %NULL.
- */
-void
-gtd_task_row_set_new_task_mode (GtdTaskRow *row,
-                                gboolean    new_task_mode)
-{
-  g_return_if_fail (GTD_IS_TASK_ROW (row));
-
-  if (row->new_task_mode != new_task_mode)
-    {
-      row->new_task_mode = new_task_mode;
-
-      if (new_task_mode)
-        {
-          gtk_stack_set_visible_child_name (GTK_STACK (row->stack), "new");
-          gtd_task_row_reveal (row);
-        }
-      else
-        {
-          gtk_stack_set_visible_child_name (GTK_STACK (row->stack), "task");
-        }
-
-      g_object_notify (G_OBJECT (row), "new-task-mode");
-    }
+  return g_object_new (GTD_TYPE_TASK_ROW,
+                       "task", task,
+                       NULL);
 }
 
 /**
@@ -736,71 +715,65 @@ gtd_task_row_set_task (GtdTaskRow *row,
 {
   g_return_if_fail (GTD_IS_TASK_ROW (row));
 
-  if (g_set_object (&row->task, task))
+  if (!g_set_object (&row->task, task))
+    return;
+
+  if (task)
     {
-      if (task)
-        {
-          gtk_label_set_label (row->task_list_label, gtd_task_list_get_name (gtd_task_get_list (task)));
+      gtk_label_set_label (row->task_list_label, gtd_task_list_get_name (gtd_task_get_list (task)));
 
-          g_object_bind_property (task,
-                                  "title",
-                                  row->title_entry,
-                                  "text",
-                                  G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+      g_signal_handlers_block_by_func (row->done_check, complete_check_toggled_cb, row);
 
-          g_object_bind_property (task,
-                                  "title",
-                                  row->title_label,
-                                  "label",
-                                  G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+      g_object_bind_property (task,
+                              "title",
+                              row->title_entry,
+                              "text",
+                              G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
 
-          g_object_bind_property (task,
-                                  "complete",
-                                  row->done_check,
-                                  "active",
-                                  G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+      g_object_bind_property (task,
+                              "complete",
+                              row->done_check,
+                              "active",
+                              G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
-          g_object_bind_property (task,
-                                  "ready",
-                                  row->task_loading_spinner,
-                                  "visible",
-                                  G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
+      g_object_bind_property_full (task,
+                                   "due-date",
+                                   row->task_date_label,
+                                   "label",
+                                   G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE,
+                                   gtd_task_row__date_changed_binding,
+                                   NULL,
+                                   row,
+                                   NULL);
 
-          g_object_bind_property_full (task,
-                                       "due-date",
-                                       row->task_date_label,
-                                       "label",
-                                       G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE,
-                                       gtd_task_row__date_changed_binding,
-                                       NULL,
-                                       row,
-                                       NULL);
+      /*
+       * Here we generate a false callback call just to reuse the method to
+       * sync the initial state of the priority icon.
+       */
+      priority_changed_cb (row, NULL, G_OBJECT (task));
+      g_signal_connect_swapped (task,
+                                "notify::priority",
+                                G_CALLBACK (priority_changed_cb),
+                                row);
 
-          /*
-           * Here we generate a false callback call just to reuse the method to
-           * sync the initial state of the priority icon.
-           */
-          gtd_task_row__priority_changed_cb (row, NULL, G_OBJECT (task));
-          g_signal_connect_swapped (task,
-                                    "notify::priority",
-                                    G_CALLBACK (gtd_task_row__priority_changed_cb),
-                                    row);
+      complete_changed_cb (row, NULL, task);
+      g_signal_connect_swapped (task,
+                                "notify::complete",
+                                G_CALLBACK (complete_changed_cb),
+                                row);
 
-          complete_changed_cb (row, NULL, task);
-          g_signal_connect_swapped (task,
-                                    "notify::complete",
-                                    G_CALLBACK (complete_changed_cb),
-                                    row);
+      depth_changed_cb (row, NULL, task);
+      g_signal_connect_swapped (task,
+                                "notify::depth",
+                                G_CALLBACK (depth_changed_cb),
+                                row);
 
-          depth_changed_cb (row, NULL, task);
-          g_signal_connect_swapped (task,
-                                    "notify::depth",
-                                    G_CALLBACK (depth_changed_cb),
-                                    row);
-        }
+      g_signal_handlers_unblock_by_func (row->done_check, complete_check_toggled_cb, row);
 
-      g_object_notify (G_OBJECT (row), "task");
+      gtd_edit_pane_set_task (GTD_EDIT_PANE (row->edit_panel), task);
     }
+
+  g_object_notify (G_OBJECT (row), "task");
 }
 
 /**
@@ -819,6 +792,22 @@ gtd_task_row_set_list_name_visible (GtdTaskRow *row,
   gtk_widget_set_visible (GTK_WIDGET (row->task_list_label), show_list_name);
 }
 
+/**
+ * gtd_task_row_set_due_date_visible:
+ * @row: a #GtdTaskRow
+ * @show_due_date: %TRUE to show the due, %FALSE to hide it
+ *
+ * Sets @row's due date label visibility to @show_due_date.
+ */
+void
+gtd_task_row_set_due_date_visible (GtdTaskRow *row,
+                                   gboolean    show_due_date)
+{
+  g_return_if_fail (GTD_IS_TASK_ROW (row));
+
+  gtk_widget_set_visible (GTK_WIDGET (row->task_date_label), show_due_date);
+}
+
 /**
  * gtd_task_row_reveal:
  * @row: a #GtdTaskRow
@@ -881,8 +870,22 @@ gtd_task_row_set_handle_subtasks (GtdTaskRow *self,
   self->handle_subtasks = handle_subtasks;
 
   gtk_widget_set_visible (self->dnd_box, handle_subtasks);
+  gtk_widget_set_visible (self->dnd_event_box, handle_subtasks);
   depth_changed_cb (self, NULL, self->task);
 
+  if (handle_subtasks)
+    {
+      gtk_drag_source_set (self->header_event_box,
+                           GDK_BUTTON1_MASK,
+                           NULL,
+                           0,
+                           GDK_ACTION_MOVE);
+    }
+  else
+    {
+      gtk_drag_source_unset (self->header_event_box);
+    }
+
   g_object_notify (G_OBJECT (self), "handle-subtasks");
 }
 
@@ -898,6 +901,8 @@ void
 gtd_task_row_set_active (GtdTaskRow *self,
                          gboolean    active)
 {
+  GtkStyleContext *context;
+
   g_return_if_fail (GTD_IS_TASK_ROW (self));
 
   if (self->active == active)
@@ -905,28 +910,36 @@ gtd_task_row_set_active (GtdTaskRow *self,
 
   self->active = active;
 
+  /* Update the arrow icon */
+  context = gtk_widget_get_style_context (self->toggle_button);
+
   if (active)
-    {
-      if (self->new_task_mode)
-        {
-          gtk_stack_set_visible_child_name (self->new_task_stack, "entry");
-          gtk_widget_grab_focus (GTK_WIDGET (self->new_task_entry));
-
-          g_signal_emit (self, signals[EXIT], 0);
-        }
-      else
-        {
-          gtk_stack_set_visible_child_name (self->task_stack, "title");
-          gtk_widget_grab_focus (GTK_WIDGET (self->title_entry));
-
-          g_signal_emit (self, signals[ENTER], 0);
-        }
-    }
+    gtk_style_context_add_class (context, "active");
   else
-    {
-      if (self->new_task_mode)
-        gtk_stack_set_visible_child_name (self->new_task_stack, "label");
-      else
-        gtk_stack_set_visible_child_name (self->task_stack, "label");
-    }
+    gtk_style_context_remove_class (context, "active");
+
+  /* And the listbox */
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->edit_panel_revealer), active);
+
+  g_signal_emit (self, active ? signals[ENTER] : signals[EXIT], 0);
+}
+
+void
+gtd_task_row_set_sizegroups (GtdTaskRow   *self,
+                             GtkSizeGroup *name_group,
+                             GtkSizeGroup *date_group)
+{
+  gtk_size_group_add_widget (name_group, GTK_WIDGET (self->task_list_label));
+  gtk_size_group_add_widget (name_group, GTK_WIDGET (self->task_date_label));
+}
+
+gint
+gtd_task_row_get_x_offset (GtdTaskRow *self)
+{
+  g_return_val_if_fail (GTD_IS_TASK_ROW (self), -1);
+
+  if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
+    return gtk_widget_get_allocated_width (GTK_WIDGET (self)) - self->clicked_x;
+  else
+    return self->clicked_x;
 }
