@@ -32,11 +32,6 @@ G_DECLARE_FINAL_TYPE (GtdTaskRow, gtd_task_row, GTD, TASK_ROW, GtkListBoxRow)
 
 GtkWidget*                gtd_task_row_new                      (GtdTask             *task);
 
-gboolean                  gtd_task_row_get_new_task_mode        (GtdTaskRow          *row);
-
-void                      gtd_task_row_set_new_task_mode        (GtdTaskRow          *row,
-                                                                 gboolean             new_task_mode);
-
 GtdTask*                  gtd_task_row_get_task                 (GtdTaskRow          *row);
 
 void                      gtd_task_row_set_task                 (GtdTaskRow          *row,
@@ -45,6 +40,9 @@ void                      gtd_task_row_set_task                 (GtdTaskRow
 void                      gtd_task_row_set_list_name_visible    (GtdTaskRow          *row,
                                                                  gboolean             show_list_name);
 
+void                      gtd_task_row_set_due_date_visible     (GtdTaskRow          *row,
+                                                                 gboolean             show_due_date);
+
 void                      gtd_task_row_reveal                   (GtdTaskRow          *row);
 
 void                      gtd_task_row_destroy                  (GtdTaskRow          *row);
@@ -59,6 +57,12 @@ gboolean                  gtd_task_row_get_active               (GtdTaskRow
 void                      gtd_task_row_set_active               (GtdTaskRow          *self,
                                                                  gboolean             active);
 
+void                      gtd_task_row_set_sizegroups           (GtdTaskRow          *self,
+                                                                 GtkSizeGroup        *name_group,
+                                                                 GtkSizeGroup        *date_group);
+
+gint                      gtd_task_row_get_x_offset             (GtdTaskRow          *self);
+
 G_END_DECLS
 
 #endif /* GTD_TASK_ROW_H */
