@@ -409,7 +409,7 @@ gtd_list_selector_grid_init (GtdListSelectorGrid *self)
   widget = GTK_WIDGET (self);
   flowbox = GTK_FLOW_BOX (self);
 
-  gtk_flow_box_set_max_children_per_line (flowbox, 100);
+  gtk_flow_box_set_max_children_per_line (flowbox, 9);
   gtk_flow_box_set_selection_mode (flowbox, GTK_SELECTION_NONE);
   gtk_flow_box_set_column_spacing (flowbox, 12);
   gtk_flow_box_set_row_spacing (flowbox, 12);
@@ -417,7 +417,7 @@ gtd_list_selector_grid_init (GtdListSelectorGrid *self)
   gtk_container_set_border_width (GTK_CONTAINER (self), 12);
   gtk_widget_set_hexpand (widget, TRUE);
   gtk_widget_set_vexpand (widget, TRUE);
-  gtk_widget_set_halign (widget, GTK_ALIGN_START);
+  gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (widget, GTK_ALIGN_START);
   gtk_widget_show_all (widget);
 }
